/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btdebug.h,v 1.5 2002/10/31 10:39:14 kds Exp $

   BTDEBUG - Debug primitives

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/


#ifndef _AFFIX_BTDEBUG_H
#define _AFFIX_BTDEBUG_H

#include <affix/bluetooth.h>

#ifdef CONFIG_AFFIX_DEBUG
#if 0
static void __get_user_X(void)
{
	return;
}
static __inline__ void __put_user_bad(void)
{
	return;
}
#endif
#endif

extern __u32	affix_dbmask;

#ifdef CONFIG_AFFIX_DEBUG
	#define DBPRT(dbtype, args...) \
	{ \
		if ((affix_dbmask & FILEBIT) && (affix_dbmask & (dbtype))) { \
			if (affix_dbmask & DBFNAME) \
				printk(KERN_DEBUG __FUNCTION__ ": " args); \
			else \
				printk(KERN_DEBUG args); \
		} \
	}

	#define DBPRTF(dbtype, args...) \
	{ \
		if ((affix_dbmask & FILEBIT) && (affix_dbmask & (dbtype))) \
			printk(KERN_DEBUG __FUNCTION__ ": " args); \
	}


	#define DBPRTPKT(dbtype, data, len) \
	{ \
		if( (affix_dbmask & FILEBIT) && (affix_dbmask & DBDUMP) && \
		    (affix_dbmask & (dbtype)) ) \
			printk_hexdump(data, len); \
	}
	
	#define DBPARSEHCI(dbtype, data, len, pkttype, dir) \
	{ \
		if( (affix_dbmask & FILEBIT) && (affix_dbmask & DBPARSE) && \
		    (affix_dbmask & (dbtype)) ) \
			printk_parse_hci(data, len, pkttype, dir); \
	}

	#define DBDUMPCHAR(dbtype, data, len) \
	{ \
		if( (affix_dbmask & FILEBIT) && (affix_dbmask & DBCHARDUMP) && \
		    (affix_dbmask & (dbtype)) ) \
			printk_chardump(data, len); \
	}

	#define DBPARSERFCOMM(dbtype, data, len, dir) \
	{ \
		if( (affix_dbmask & FILEBIT) && (affix_dbmask & DBPARSE) && \
		    (affix_dbmask & (dbtype)) ) \
			printk_parse_rfcomm(data, len, dir); \
	}

#define BTASSERT(expr, func) \
if(!(expr)) { \
        printk( "Assertion failed! %s,%s,%s,line=%d\n",\
        #expr,__FILE__,__FUNCTION__,__LINE__); \
        ##func}


#define DCODE(args...)	{args}

#else /* CONFIG_AFFIX_DEBUG */

#define DBPRT(dbtype, format, args...) 
#define DBPRTF(dbtype, args...) 
#define DBPRTPKT(dbtype, data, len)
#define DBPARSEHCI(dbtype, data, len, pkttype, dir)
#define DBPARSERFCOMM(dbtype, data, len, dir)
#define DBDUMPCHAR(dbtype, data, len)

#define BTASSERT(expr, func)
#define DCODE(args...)

#endif /* CONFIG_AFFIX_DEBUG */

#define DBFENTER	DBPRTF(DBFUNC, "Entered\n")
#define DBFEXIT		DBPRTF(DBFUNC, "Exited\n")

#define BTERROR(args...)   printk(KERN_ERR __FUNCTION__ ": " args)
#define BTWARN(args...) printk(KERN_WARNING __FUNCTION__ ": " args)
#define BTNOTICE(args...) printk(KERN_INFO __FUNCTION__ ": " args)
#define BTINFO(args...) printk(KERN_INFO __FUNCTION__ ": " args)
#define BTDEBUG(args...) printk(KERN_DEBUG __FUNCTION__ ": " args)


//      For packet parsing
typedef volatile enum
{
        TO_HOSTCTRL = 0,
        FROM_HOSTCTRL
} pkt_dir;

/* some functions to help... */
void printk_hexdump(const __u8 *p, __u32 len);
void printk_chardump(const __u8 *p, int len);

void printk_parse_hci(const __u8 *p, int data_size, int type, pkt_dir dir); 
void printk_parse_rfcomm(const __u8 *p, int len, pkt_dir dir);


#endif
