/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: hci_mgr.h,v 1.14 2002/11/12 09:54:50 kds Exp $

   HCI Manager

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
                Imre Deak <ext-imre.deak@nokia.com>
*/		

#ifndef _HCI_HCICTL_H
#define _HCI_HCICTL_H

#include <affix/hci.h>


extern int	affix_ctl_mode;


/* Manager Section */
int hci_start_manager(void);
void hci_stop_manager(void);

/* some commands */
int hci_state_change(hci_struct *hci, int event);
int hci_connect_req(hci_con *con);
int hci_auth_req(hci_con *con);
int hci_disconnect_req(hci_con *con, __u8 reason);


/* Neighbour Device management */
btdev_struct *btdev_lookup_bda(BD_ADDR *bda);
btdev_struct *btdev_create(BD_ADDR *bda);
void btdev_destroy(btdev_struct *btdev);
void __btdev_destroy(btdev_struct *btdev);
void btdev_flush(void);

static inline void btdev_hold(btdev_struct *btdev)
{
	atomic_inc(&btdev->refcnt);
}

static inline void btdev_put(btdev_struct *btdev)
{
	if (atomic_dec_and_test(&btdev->refcnt))
		btdev_destroy(btdev);
}

static inline void __btdev_put(btdev_struct *btdev)
{
	if (atomic_dec_and_test(&btdev->refcnt))
		__btdev_destroy(btdev);
}


/*
 * Security block
 */
int affix_add_pin(struct PIN_Code *pin);
int affix_remove_pin(BD_ADDR *bda);
int affix_add_key(struct link_key *key);
int affix_remove_key(BD_ADDR *bda);


int hpf_connect_cfm(hci_con *con, int status);
int hpf_disconnect_ind(hci_con *con);
int hpf_data_ind(hci_con *con, struct sk_buff *skb);

int hpf_get_devs(int *devs);
int hpf_start_dev(hci_struct *hci);

#endif
